package net.knarcraft.placeholdersigns.handler;

import net.knarcraft.placeholdersigns.container.SignLineChangeRequest;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.Map;

/**
 * A class for keeping track of placeholder sign requests
 */
public class PlaceholderSignRequestHandler {

    private final Map<Player, SignLineChangeRequest> signChangeRequests;
    private final Map<Player, Boolean> signViewRequests;

    /**
     * Instantiates a new placeholder sign request handler
     */
    public PlaceholderSignRequestHandler() {
        this.signChangeRequests = new HashMap<>();
        this.signViewRequests = new HashMap<>();
    }

    /**
     * Registers a sign change request
     *
     * <p>A sign change request is basically the result of running the editSign command, which must be stored until the
     * player clicks a sign.</p>
     *
     * @param request <p>The sign change request to register</p>
     */
    public void addSignChangeRequest(@NotNull SignLineChangeRequest request) {
        signChangeRequests.put(request.player(), request);
    }

    /**
     * Gets a sign change request
     *
     * @param player <p>The player to get the request for</p>
     * @return <p>The sign change request, or null if not found</p>
     */
    @Nullable
    public SignLineChangeRequest getSignChangeRequest(@NotNull Player player) {
        return signChangeRequests.remove(player);
    }

    /**
     * Registers a sign view request
     *
     * @param player <p>The player requesting to view a sign</p>
     * @param raw    <p>Whether to display the raw text (visible formatting codes)</p>
     */
    public void addSignViewRequest(@NotNull Player player, boolean raw) {
        signViewRequests.put(player, raw);
    }

    /**
     * Checks whether the given player has a sign view request
     *
     * @param player <p>The player to check</p>
     * @return <p>True if the player has requested to view a sign</p>
     */
    @Nullable
    public Boolean getSignViewRequest(@NotNull Player player) {
        if (signViewRequests.containsKey(player)) {
            return signViewRequests.remove(player);
        } else {
            return null;
        }
    }

}
