package net.knarcraft.placeholdersigns.listener;

import me.clip.placeholderapi.PlaceholderAPI;
import net.knarcraft.placeholdersigns.PlaceholderSigns;
import net.knarcraft.placeholdersigns.container.PlaceholderSign;
import net.knarcraft.placeholdersigns.handler.PlaceholderSignHandler;
import org.bukkit.Location;
import org.bukkit.block.sign.Side;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.SignChangeEvent;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Map;

/**
 * A listener for signs being changed to contain on or more placeholders
 */
public class SignTextListener implements Listener {

    @EventHandler(priority = EventPriority.MONITOR, ignoreCancelled = true)
    public void onSignCreateOrEdit(@NotNull SignChangeEvent event) {
        // Only check for placeholders if the player is allowed to
        if (!event.getPlayer().hasPermission("placeholdersigns.placeholder")) {
            return;
        }

        String[] lines = event.getLines();
        Map<Integer, String> placeholders = new HashMap<>();

        // Register any lines PlaceholderAPI wants to change
        for (int i = 0; i < lines.length; i++) {
            String line = lines[i];
            String replaced = PlaceholderAPI.setPlaceholders(null, line);
            if (line.equalsIgnoreCase(replaced)) {
                continue;
            }
            placeholders.put(i, line);
        }

        updatePlaceholders(placeholders, event);
    }

    /**
     * Updates placeholders as necessary
     *
     * @param placeholders <p>The placeholders parsed from the lines</p>
     * @param event        <p>The triggered sign change event</p>
     */
    private void updatePlaceholders(@NotNull Map<Integer, String> placeholders,
                                    @NotNull SignChangeEvent event) {
        Location location = event.getBlock().getLocation();
        PlaceholderSignHandler signHandler = PlaceholderSigns.getInstance().getSignHandler();
        PlaceholderSign existingSign = signHandler.getFromLocation(location);

        if (!placeholders.isEmpty() && existingSign == null) {
            Map<Side, Map<Integer, String>> placeholderSide = new HashMap<>();
            placeholderSide.put(event.getSide(), placeholders);

            // Register a new placeholder sign
            PlaceholderSign placeholderSign = new PlaceholderSign(event.getBlock().getLocation(), placeholderSide);
            signHandler.registerSign(placeholderSign);
        } else if (!placeholders.isEmpty()) {
            // Overwrite the placeholders of the existing placeholder sign
            Map<Integer, String> existing = existingSign.placeholders().computeIfAbsent(event.getSide(), k -> new HashMap<>());
            existing.putAll(placeholders);
            signHandler.save();
        }
    }

}
